# python3-opsi
# This code is part of the opsi.org project
# http://opsi.org
#
# Copyright (c) uib gmbh (www.uib.de)
# This sourcecode is owned by the uib gmbh, D55118 Mainz, Germany
# and published under the Terms of the GNU Affero General Public License.
# Text of the AGPL: http://www.gnu.org/licenses/agpl-3.0-standalone.html

try:
    import ConfigParser
except:
    import configparser
from OPSI.Util.File import IniFile
from OPSI.Util.File import TextFile
from OPSI.Backend.BackendManager import BackendManager
from OPSI.Backend.JSONRPC import JSONRPCBackend
from OPSI.Util import md5sum
from hashlib import md5
try:
    from urlparse import urlparse
except:
    from urllib.parse import urlparse
from os.path import splitext, basename
from ipaddress import ip_address
import errno
import platform
import re, xml.dom.minidom, os, gzip

if not bootimageVersion or (int(bootimageVersion) < 20140704):
    raise Exception(u"This product requires opsi-linux-bootimage version >= 20140704")
try:
    if not backend.backend_info()['opsiVersion'].startswith('4.'):
        raise ValueError('No opsi 4.x backend present')
except Exception:
    raise Exception(u"This is a opsi 4.x product")

isOpsi43 = False
if backend.backend_info()['opsiVersion'].startswith('4.3'):
    isOpsi43 = True

if isOpsi43:
    show_message(u"Installing Product: %s" % (productId), 10)

opsiVersion = backend.backend_info()['opsiVersion']

#include opsi setup library
depot.copy(SCRIPT_PATH+'/opsisetuplib.py','/tmp')
exec(open('/tmp/opsisetuplib.py').read())

#include opsi setup library
depot.copy(SCRIPT_PATH+'/opsipreparelib.py','/tmp')
exec(open('/tmp/opsipreparelib.py').read())


initOpsiSetupLib()
olimode = False
if "opsi-local-image" in productId:
    logger.notice(u"Running in local image mode")
    olimode = True
    checkModules('local_imaging')
    use_lvm = False
    encrypt_lvm = False
################ start dist specific part 1 #################################################################################
logger.notice(u"%s.py" % productId)
logger.notice(u"##############################################################")
productVersion =''
packageVersion =''
for i in range(len(productOnDepots)):
    if (productOnDepots[i]['productId'] == productId):
        productVersion = productOnDepots[i]['productVersion']
        packageVersion = productOnDepots[i]['packageVersion']
    break
logger.notice(u"Installing Product: %s_%s-%s on %s" % (productId,productVersion,packageVersion,clientId))
scriptMessageSubject.setMessage(u"Installing Product: %s_%s-%s on %s" % (productId,productVersion,packageVersion,clientId))
logger.notice(u"##############################################################")

release = productProperties.get('release', 'jammy')

addAllNetworkDevices = False
if (str(productProperties.get('add_all_network_devices_to_netplan', '')).lower() == 'true'):
    addAllNetworkDevices = True

netplan_use_mac_identifier = False
if (str(productProperties.get('netplan_use_mac_identifier', '')).lower() == 'true'):
    netplan_use_mac_identifier = True

if "ubuntu" in productId:
    online_repository = productProperties.get('online_repository', 'http://de.archive.ubuntu.com/ubuntu')
else:
    online_repository = productProperties.get('online_repository', 'http://ftp.debian.org')

project_quota = False
if (str(productProperties.get('project_quota', 'true')).lower() == 'true'):
    project_quota = True

scriptMessageSubject.setMessage("Installing %s %s..." % (productId,release))
logger.notice(u"Installing %s %s..." % (productId, release))


################ end dist specific part 1 #################################################################################

if olimode:
    backupAfterInstall = True
    if (str(productProperties.get('backup_after_install', 'true')).lower() == 'false'):
        backupAfterInstall = False
        logger.notice(u"backupAfterInstall = False")
    else:
        logger.notice(u"backupAfterInstall = True")

    setupAfterInstall = productPropertyValues.get('setup_after_install', '')
    proxy = productProperties.get('proxy','')
    no_proxy = productProperties.get('no_proxy','')

    install_opsi_client_agent = False
    if (str(productProperties.get('install_opsi-client-agent', 'true')).lower() == 'true'):
        install_opsi_client_agent = True

    wget_and_execute = productProperties.get('wget_and_execute', '')
    wget_and_execute = wget_and_execute.strip()

    # get multiDiskMode and backup_partition_on_same_disk from opsi-local-image-prepare
    multiDiskMode  = '0'
    backup_partition_on_same_disk = True
    if hasattr(backend, "productPropertyState_getValues"):
        # opsi >= 4.3
        for property_id, values in (
            backend.productPropertyState_getValues(product_ids=['opsi-local-image-prepare'], object_ids=[clientId], with_defaults=True)[clientId]['opsi-local-image-prepare']
            .items()
        ):
            logger.info("Found product property: %s=%r", property_id, values)
            if property_id == 'multi_disk_mode':
                multiDiskMode  = values[0]
            if property_id == 'backup_partition_on_same_disk':
                backup_partition_on_same_disk = values[0]
    else:
        backend.backend_setOptions( { 'addProductPropertyStateDefaults': True } )
        pps = backend.productPropertyState_getObjects(objectId = clientId, productId = 'opsi-local-image-prepare',propertyId = 'multi_disk_mode')
        if pps:
            for state in pps:
                myvalues = state.getValues()
                for key in myvalues:
                    multiDiskMode  = key
        pps = backend.productPropertyState_getObjects(objectId = clientId, productId = 'opsi-local-image-prepare',propertyId = 'backup_partition_on_same_disk')
        if pps:
            for state in pps:
                myvalues = state.getValues()
                for key in myvalues:
                    backup_partition_on_same_disk  = key
        backend.backend_setOptions( { 'addProductPropertyStateDefaults': False } )

    logger.notice("multiDiskMode: %s ; backup_partition_on_same_disk: %s" % (multiDiskMode,backup_partition_on_same_disk))

    encodedPcpatchPassword = ""


    target = '/mnt/hd'
    source = SCRIPT_PATH
    depotsource = DEPOT_MOUNT + '/' + SCRIPT_PATH
    imageDir = '/mntimg'
    filesystem = 'ext4'

# Do hardware inventory
logger.notice(u"Fetching opsi hw audit configuration")
hwconfig = backend.auditHardware_getConfig()

if isOpsi43:
    show_message(u"Running hardware inventory", 10)
logger.notice(u"Running hardware inventory")
auditHardwareOnHosts = auditHardware(config = hwconfig, hostId = clientId)

logger.notice(u"Sending hardware information to service")
backend.auditHardwareOnHost_setObsolete(clientId)
backend.auditHardwareOnHost_createObjects(auditHardwareOnHosts)

if not olimode:
    scriptMessageSubject.setMessage("Prepare disk..")
    logger.notice(u"Prepare disk...")
    if isOpsi43:
        show_message(u"Preparing disk", 20)
    prepareDisk()

################# start dist specific part 2 #################################################################################
if olimode:
    blockAlignment = True
    useRaid1 = False
    goOn = True

    useGpt = False
    inUefi = inUefiMode()
    if inUefi:
        useGpt = True

    if inUefi:
        currentbootentry = uefiGetActBootEntry()
        if currentbootentry:
            uefiNetbootLabel = currentbootentry['label']
            logger.notice('Current (net) boot entry: %s' % uefiNetbootLabel)
            backend.config_createUnicode("clientConfig.uefiNetbootLabel","clientspecific uefi firmware netboot entry label")
            backend.configState_create("clientConfig.uefiNetbootLabel", clientId, [ uefiNetbootLabel ] )
        else:
            logger.notice('No usable current (net) boot entry found')

    if useGpt:
        logger.notice(u"Will use GPT")
    else:
        logger.notice(u"Will use MBR Partitions")

    # get machine architecture
    myarchitecture = 'i386'
    myplatform = platform.machine()
    if myplatform in ['x86_64', 'AMD64', 'amd64']:
        myarchitecture = 'amd64'
        ui.getMessageBox().addText(u"Installing 64 Bit system....\n")
        logger.notice(u"Installing 64 Bit system....\n")
    else:
        ui.getMessageBox().addText(u"Installing 32 Bit system\n")
        logger.notice(u"Installing 32 Bit system....\n")

    systemPartitionNumber = 1
    systemPartitionLabel  = 'system'
    systemPartitionSizeM = 0
    systemPartitionLetter = "C"

    dataPartitionSizeM = 0
    dataPartitionNumber = 0
    dataPartitionLabel  = 'data'
    dataPartitionLetter = 'D'
    #dataPartitionLetter = productProperties.get('data_partition_letter', 'D')

    swapPartitionSizeM   = 4000
    winpePartitionNumber = 2
    swapPartitionSize   = "4000M"
    swapPartitionLabel  = "winpe"
    swapPartitionLetter = "X"

    backupPartitionNumber = 3
    backupPartitionCreate = False
    backupPartitionLabel  = "opsidata"

    if useGpt:
        systemPartitionNumber += 2
        winpePartitionNumber += 2
        backupPartitionNumber += 2
    logger.notice("Partition numbers are now: system=%d, data=%s, swap=%d, backup=%d" % (systemPartitionNumber, dataPartitionNumber, winpePartitionNumber, backupPartitionNumber))


    logger.notice(u"Swapoff all to avoid disk in use .....")
    execute("swapoff -a -v")
    lvmClearVolumeGroups()

    # Get harddisks
    disks = getHarddisks()
    disks = sorted(disks, key=lambda disk: disk.device)
    showDiskOrder(disks)
    logger.notice(u"Normally we using disk 0")
    diskindex = getDiskIndexFromMultiDiskMode(multiDiskMode)
    disk = disks[diskindex]
    logger.notice(u"Using disk device %s" % (disk.device))
    # check if (backup_partition_on_same_disk == False) and only one disk, what would make no sense
    # and if so fall back to (backup_partition_on_same_disk == true)
    backupdiskindex, backup_partition_on_same_disk = getBackupdiskindexFromBackupPartitionOnSameDisk(backup_partition_on_same_disk)
    logger.notice(u"backup_partition_on_same_disk: %s - backupdiskindex %s" % (backup_partition_on_same_disk,backupdiskindex))
    if (backup_partition_on_same_disk == False):
        # so we have a backup disk 
        backupPartitionNumber = 1

    logger.notice("Partition numbers are now: system=%d, data=%s, swap=%d, backup=%d" % (systemPartitionNumber, dataPartitionNumber, winpePartitionNumber, backupPartitionNumber))

    disk.setBlockAlignment(blockAlignment)
    diskSizeM = (float(disk.size)/(1024*1024))
    diskTotalSectors = disk.sectors

    checkOliPartitions(diskindex,backupdiskindex,useGpt,backup_partition_on_same_disk)

    # change partition id and boot flag
    if inUefi:
        #system
        logger.notice(u"change id of partition %s to 8300" % (getPartitionDevicePath(disk.device,systemPartitionNumber)))
        execute('/usr/sbin/sgdisk -t %d:8300 --attributes %d:set:0 -p %s' % (systemPartitionNumber, systemPartitionNumber, disk.device))
        execute('/usr/sbin/sgdisk --info=%d %s' % (systemPartitionNumber, disk.device))
        #execute('sfdisk --change-id %s %d 07' % (disk.device,systemPartitionNumber))
        #winpe/swap
        logger.notice(u"change id of partition %s to 8200" % (getPartitionDevicePath(disk.device,winpePartitionNumber)))
        execute('/usr/sbin/sgdisk -t %d:8200 --attributes %d:clear:62 -p %s' % (winpePartitionNumber, winpePartitionNumber, disk.device))
        execute('/usr/sbin/sgdisk --info=%d %s' % (winpePartitionNumber, disk.device))
        deleteAllEntriesFromEFIBootmgr('opsitempwinpe')
        deleteAllEntriesFromEFIBootmgr('Windows Boot Manager')
        logger.notice(u"setting opsitempwinpe to the top of uefi boot order")
        execute('/bin/efibootmgr -c --disk %s --part %s -L opsitempwinpe --loader \\\\EFI\\\\boot\\\\bootx64.efi' % (disk.device, winpePartitionNumber))
        execute('/bin/efibootmgr -v')
    else:
        logger.notice(u"change id of partition %s to 83" % (getPartitionDevicePath(disk.device,systemPartitionNumber)))
        execute('sfdisk --change-id %s %d 83' % (disk.device,systemPartitionNumber))
        logger.notice(u"Activate (set boot flag) on partition %s%d" % (disk.device,systemPartitionNumber))
        execute('sfdisk -A %s %d' % (disk.device,systemPartitionNumber))
        logger.notice(u"change id of partition %s to 82" % (getPartitionDevicePath(disk.device,winpePartitionNumber)))
        execute('sfdisk --change-id %s %d 82' % (disk.device,winpePartitionNumber))
        exitcode = os.system('%s %s >> /dev/null 2>&1' % (which('partprobe'), disk.device))
        execute('sfdisk -l')


    # create mount point before loop
    checkClearMountpoint("%s/proc" % target, "none", "none")
    checkClearMountpoint("%s/sys" % target, "none", "none")
    checkClearMountpoint("%s/dev" % target, "none", "none")
    checkClearMountpoint("%s" % target, "none", "none")
    execute("swapoff -a")
    checkCreateDir(target)

    disk.setBlockAlignment(blockAlignment)
    scriptMessageSubject.setMessage(u"Using harddisk %s (%0.0f MB)." % (disk.device, (float(disk.size)/(1024*1024))))

    # Check disk size
    if (disk.size < 18000*1024*1024):
        # Disk smaller than 18000 MB => give up
        raise Exception(u"Disk to small")

    if not useGpt:
        # Get current partitions
        partitions = disk.getPartitions()

        if not partitions:
            # No partition found on harddisk
            scriptMessageSubject.setMessage(u"Found no partitions on %s" % disk.device)


    # System partition will be deleted => delete all netboot product states
    deletePocs = []
    for poc in backend.productOnClient_getObjects(clientId = clientId):
        if ((poc.productType == 'NetbootProduct') and (poc.productId != productId)) or \
        ((poc.productType == 'LocalbootProduct') and (poc.installationStatus == 'not_installed') and (poc.actionRequest == 'none')):
            deletePocs.append(poc)
    backend.productOnClient_deleteObjects(deletePocs)

    swapPartitionDevicePath = getPartitionDevicePath(disk.device, winpePartitionNumber)
    systemPartitionDevicePath = getPartitionDevicePath(disk.device, systemPartitionNumber)

    # Create ext4 filesystem on system partition
    testPartitionEx(disk.device, systemPartitionNumber, useGpt)
    checkClearPartition("none", disk.device, systemPartitionNumber)
    createFilesystemEx("system", systemPartitionNumber, "system", "ext4", disk.device , useGpt)

    # create swap
    logger.notice(u"Create swap and swapon swap partition")
    testPartitionEx(disk.device, winpePartitionNumber, useGpt)
    checkClearPartition("none", disk.device, winpePartitionNumber)
    try:
        execute("mkswap -L %s %s" % (swapPartitionLabel, swapPartitionDevicePath) )
    except:
        logger.notice(u"mkswap failed - retrying....")
        execute('/bin/sleep 5')
        execute('blkid')
        execute("mkswap -L %s %s" % (swapPartitionLabel, swapPartitionDevicePath) )


    #resultlist = execute(u'blkid -o value -s UUID %s' % (systemPartitionDevicePath) )
    #for resultline in resultlist:
    #       try:
    #               systemPartitionUUID = resultline.strip()
    #       except:
    #               pass
    #systemPartitionDevicePathUUID = "UUID=%s" % (systemPartitionUUID)
    #resultlist = execute(u'blkid -o value -s UUID %s' % (swapPartitionDevicePath) )
    #for resultline in resultlist:
    #       try:
    #               swapPartitionUUID = resultline.strip()
    #       except:
    #               pass
    #swapPartitionDevicePathUUID = "UUID=%s" % (swapPartitionUUID)

    systemPartitionUUID = getUuidFromPartitionDevice(systemPartitionDevicePath)
    systemPartitionDevicePathUUID = "UUID=%s" % (systemPartitionUUID)
    swapPartitionUUID = getUuidFromPartitionDevice(swapPartitionDevicePath)
    swapPartitionDevicePathUUID = "UUID=%s" % (swapPartitionUUID)

    logger.notice(u"systemPartitionDevicePath = %s" % systemPartitionDevicePath)
    logger.notice(u"swapPartitionDevicePath = %s" % swapPartitionDevicePath)

# Mount partition
if int(swapPartitionSizeM) > 0:
    logger.notice(u"mount swap partition")
    execute("swapon %s" % (swapPartitionDevicePath) )
logger.notice(u"mount system partition")
checkCreateDir(target)
checkClearMountpoint(target, disk.device, systemPartitionNumber, systemPartitionDevicePath)
checkClearPartition(target, disk.device, systemPartitionNumber, systemPartitionDevicePath)
checkMount(target, disk.device, systemPartitionNumber, "ext4", systemPartitionDevicePath)
if not olimode:
    if bootPartitionDevicePath:
        checkCreateDir(target+"/boot")
        checkClearMountpoint("%s/boot" % target, disk.device, bootPartitionNumber, bootPartitionDevicePath)
        checkClearPartition(target+"/boot", disk.device, bootPartitionNumber, bootPartitionDevicePath)
        checkMount(target+"/boot", disk.device, bootPartitionNumber, "ext4", bootPartitionDevicePath)

if myarchitecture == 'amd64':
    #depot.copy("%s/%s/64bit/pkgdetails" % (source, release), "/usr/lib/debootstrap/")
    copy("%s/pkgdetails/64bit/pkgdetails" % (depotsource), "/usr/lib/debootstrap/")
    if not os.path.isfile("/usr/lib/debootstrap/pkgdetails"):
        checkCreateDir("/usr/lib/debootstrap")
        depot.copy(SCRIPT_PATH+'/pkgdetails/64bit/pkgdetails','/usr/lib/debootstrap/')
else:
    #depot.copy("%s/%s/32bit/pkgdetails" % (source, release), "/usr/lib/debootstrap/")
    copy("%s/pkgdetails/32bit/pkgdetails" % (depotsource), "/usr/lib/debootstrap/")
    if not os.path.isfile("/usr/lib/debootstrap/pkgdetails"):
        checkCreateDir("/usr/lib/debootstrap")
        depot.copy(SCRIPT_PATH+'/pkgdetails/32bit/pkgdetails','/usr/lib/debootstrap/')
    #depot.copy("%s/pkgdetails/32bit/pkgdetails" % (source), "/usr/lib/debootstrap/")
execute("chmod u+x /usr/lib/debootstrap/pkgdetails")

proxy = productProperties.get('proxy','')
no_proxy = productProperties.get('no_proxy','')
proxy_bypass_in_opsiscript = False
if (str(productProperties.get('proxy_bypass_in_opsiscript', 'true')).lower() == 'true'):
    proxy_bypass_in_opsiscript = True

no_symlink_release = ["noble"]

if isOpsi43:
    show_message(u"Installing base system ....", 30)
scriptMessageSubject.setMessage("Installing base system .... ")
logger.notice(u"Installing base system...")
if "ubuntu" in productId:
    extractor = "--extractor=ar "
else:
    extractor = ""
if release in no_symlink_release:
    debootstrap_options = "--no-merged-usr "
else:
    debootstrap_options = ""
if not os.path.exists("/usr/share/debootstrap/scripts/noble"):
    logger.notice("noble debootstrap script not found, copy from gutsy")
    execute("cp /usr/share/debootstrap/scripts/gutsy /usr/share/debootstrap/scripts/noble")
if proxy:
    cmd="http_proxy=%s %s --verbose %s --no-check-gpg --arch %s %s--include=quota %s %s %s" % (proxy, which('debootstrap'),debootstrap_options, myarchitecture, extractor, release, target, online_repository)
else:
    cmd="%s --verbose %s --no-check-gpg --arch %s %s--include=quota %s %s %s" % (which('debootstrap'), debootstrap_options, myarchitecture, extractor, release, target, online_repository)

try:
    execute(cmd, timeout=1000)
except:
    if isOpsi43:
        show_message(u"debootstrap failed - retrying....", 30)
    logger.notice(u"debootstrap failed - retrying....")
    execute('/bin/sleep 10')
    try:
        execute(cmd, timeout=1000)
    except:
        if isOpsi43:
            show_message(u"debootstrap failed - retrying....", 30)
        logger.notice(u"debootstrap failed - retrying....")
        execute('/bin/sleep 10')
        try:
            execute(cmd, timeout=1000)
        except:
            logger.notice(u"debootstrap failed - giving up....")
            raise Exception(u"debootstrap failed - giving up....")

logger.notice(u"systemPartitionDevicePath = %s" % systemPartitionDevicePath)
if int(swapPartitionSizeM) > 0:
    logger.notice(u"swapPartitionDevicePath = %s" % swapPartitionDevicePath)

if isOpsi43:
    show_message(u"Configuring base system ....", 40)
scriptMessageSubject.setMessage("Configuring base system ....")
logger.notice(u"Configuring base system ....")
if not olimode:
    logger.notice(u"Creating mtab ....")
    try:
        os.stat('/mnt/hd/etc/mtab')
    except OSError as e:
        if e.errno == errno.ENOENT:
            try:
                os.remove('/mnt/hd/etc/mtab')
            except OSError as e:
                pass
        else:
            raise e
    with open( os.path.join(target, 'etc', 'mtab'), 'w' ) as mtab:
        mtab.write("%s / ext4 rw 0 0\n" % (systemPartitionDevicePath))

logger.notice(u"Updating /etc/hosts....")
scriptMessageSubject.setMessage(u"Updating /etc/hosts....")
with open( os.path.join(target, 'etc', 'hosts'), 'a' ) as hosts:
    hosts.write("%s        %s %s\n" % (ipaddress, clientId, clientId.split('.')[0]))

if proxy:
    if proxy_bypass_in_opsiscript:
        if not bootimageVersion or (int(bootimageVersion) < 20221024):
            raise Exception(u"This product requires opsi-linux-bootimage version >= 20221024")
        logger.notice(u"trying to bypass opsi-script proxy problems")
        try:
            if ip_address(nextserver.split('/')[2].replace(':4447', '')):
                server_ip_address = nextserver.split('/')[2].replace(':4447', '')
                logger.notice("server hostname %s is an ip address" % server_ip_address)
                server_hostname = list(filter(None, execute("host {} | cut --delimiter=' ' --fields=5 | head -n 1".format(nextserver.split('/')[2].replace(':4447', '')))[0].replace('(', '').replace(')', '')))
                server_hostname = ''.join(server_hostname)
                if server_hostname:
                    logger.notice("server hostname is %s" % server_hostname[:-1])
                    if server_hostname[-1] == '.':
                        server_hostname = server_hostname[:-1]
        except ValueError:
            server_hostname = nextserver.split('/')[2].replace(':4447', '')
            logger.notice("server hostname %s is an hostname" % server_hostname)
            server_ip_address = execute("ping -c 1 {} | cut --delimiter=' ' --fields=3 | head -n 1".format(server_hostname))[0].replace('(', '').replace(')', '')
            logger.notice(" server_ip_address is %s" % server_ip_address)
        with open( os.path.join(target, 'etc', 'hosts'), 'a' ) as hosts:
            hosts.write("%s        %s %s\n" % (server_ip_address, server_hostname, server_hostname.split('.')[0]))

logger.notice(u"Creating /etc/resolv.conf....")
scriptMessageSubject.setMessage(u"Creating /etc/resolv.conf....")
copy('/etc/resolv.conf', os.path.join(target, 'etc', 'resolv.conf'))

logger.notice(u"Creating fstab ....")
with open( os.path.join(target, 'etc', 'fstab'), 'w' ) as fstab:
    fstab.write("# /etc/fstab: static file system information.\n")
    fstab.write("#\n")
    fstab.write("# <file system>   <mount point>   <type>       <options>                   <dump>  <pass>\n")
    fstab.write("proc              /proc           proc         defaults                     0        0\n")
    if not olimode:
        if bootPartitionNumber > 0:
            fstab.write("%s     /boot            ext4        defaults,errors=remount-ro  0        1\n" % (bootPartitionDevicePathUUID))
    if inUefi:
        fstab.write("%s     /boot/efi   vfat    defaults     0   1\n" % getPartitionDevicePath(disk.device, 1))
        fstab.write("efivarfs /sys/firmware/efi/efivars efivarfs rw,nosuid,nodev,noexec 0 0\n")
    if project_quota:
        fstab.write("%s     /               ext4          prjquota  0        1\n" % (systemPartitionDevicePathUUID))
    else:
        fstab.write("%s     /               ext4         defaults,errors=remount-ro  0        1\n" % (systemPartitionDevicePathUUID))
    if int(swapPartitionSizeM) > 0:
        fstab.write("%s     none            swap         sw                          0        0\n" % (swapPartitionDevicePathUUID))
    fstab.write("/dev/cdrom        /media/cdrom    udf,iso9660  user,noauto                  0        0\n")

logger.notice(u"Creating /etc/kernel-img.conf ....")
with open( os.path.join(target, 'etc', 'kernel-img.conf'), 'w' ) as kernelimg:
    kernelimg.write("do_symlinks = yes\n")
    kernelimg.write("relative_links = yes\n")
    kernelimg.write("do_bootloader = no\n")
    kernelimg.write("do_bootfloppy = no\n")
    kernelimg.write("do_initrd = yes\n")
    kernelimg.write("link_in_boot = no\n")
    kernelimg.write("postinst_hook = /sbin/update-grub\n")
    kernelimg.write("postrm_hook = /sbin/update-grub\n")

if isOpsi43:
    show_message(u"Creating /etc/apt/sources.list ....", 50)
logger.notice(u"Creating /etc/apt/sources.list ....")
if "ubuntu" in productId:
    with open( os.path.join(target, 'etc', 'apt', 'sources.list'), 'w' ) as sources:
        sources.write("deb %s/ %s main restricted\n" % (online_repository, release))
        sources.write("deb-src %s/ %s main restricted\n" % (online_repository, release))
        sources.write("\n")
        sources.write("## Major bug fix updates produced after the final release of the\n")
        sources.write("## distribution.\n")
        sources.write("deb %s/ %s-updates main restricted\n" % (online_repository, release))
        sources.write("deb-src %s/ %s-updates main restricted\n" % (online_repository, release))
        sources.write("\n")
        sources.write("## N.B. software from this repository is ENTIRELY UNSUPPORTED by the Ubuntu\n")
        sources.write("## team. Also, please note that software in universe WILL NOT receive any\n")
        sources.write("## review or updates from the Ubuntu security team.\n")
        sources.write("deb %s/ %s universe\n" % (online_repository, release))
        sources.write("deb-src %s/ %s universe\n" % (online_repository, release))
        sources.write("deb %s/ %s-updates universe\n" % (online_repository, release))
        sources.write("deb-src %s/ %s-updates universe\n" % (online_repository, release))
        sources.write("\n")
        sources.write("## N.B. software from this repository is ENTIRELY UNSUPPORTED by the Ubuntu\n")
        sources.write("## team, and may not be under a free licence. Please satisfy yourself as to\n")
        sources.write("## your rights to use the software. Also, please note that software in\n")
        sources.write("## multiverse WILL NOT receive any review or updates from the Ubuntu\n")
        sources.write("## security team.\n")
        sources.write("deb %s/ %s multiverse\n" % (online_repository, release))
        sources.write("deb-src %s/ %s multiverse\n" % (online_repository, release))
        sources.write("deb %s/ %s-updates multiverse\n" % (online_repository, release))
        sources.write("deb-src %s/ %s-updates multiverse\n" % (online_repository, release))
        sources.write("\n")
        sources.write("## Uncomment the following two lines to add software from the 'backports'\n")
        sources.write("## repository.\n")
        sources.write("## N.B. software from this repository may not have been tested as\n")
        sources.write("## extensively as that contained in the main release, although it includes\n")
        sources.write("## newer versions of some applications which may provide useful features.\n")
        sources.write("## Also, please note that software in backports WILL NOT receive any review\n")
        sources.write("## or updates from the Ubuntu security team.\n")
        sources.write("##deb %s/ %s-backports main restricted universe multiverse\n" % (online_repository, release))
        sources.write("# deb-src %s/ %s-backports main restricted universe multiverse\n" % (online_repository, release))
        sources.write("\n")
        sources.write("## Uncomment the following two lines to add software from Canonical's\n")
        sources.write("## 'partner' repository.\n")
        sources.write("## This software is not part of Ubuntu, but is offered by Canonical and the\n")
       
        if release == "focal" or release == "jammy":
                sources.write("## respective vendors as a service to Ubuntu users.\n")
                sources.write("deb http://archive.canonical.com/ubuntu/ %s partner\n" % (release))
                sources.write("# deb-src http://archive.canonical.com/ubuntu/ maverick partner\n")
                sources.write("\n")
                sources.write("## This software is not part of Ubuntu, but is offered by third-party\n")
        sources.write("## developers who want to ship their latest software.\n")
        sources.write("#deb http://extras.ubuntu.com/ubuntu/ %s main\n" % (release))
        sources.write("#deb-src http://extras.ubuntu.com/ubuntu/ %s main\n" % (release))
        sources.write("\n")
        sources.write("deb %s/ %s-security main restricted\n" % (online_repository, release))
        sources.write("deb-src %s/ %s-security main restricted\n" % (online_repository, release))
        sources.write("deb %s/ %s-security universe\n" % (online_repository, release))
        sources.write("deb-src %s/ %s-security universe\n" % (online_repository, release))
        sources.write("deb %s/ %s-security multiverse\n" % (online_repository, release))
        sources.write("deb-src %s/ %s-security multiverse\n" % (online_repository, release))
        #sources.write("deb %s ./" % opsi_online_repository
else:
    with open( os.path.join(target, 'etc', 'apt', 'sources.list'), 'w' ) as sources:
        if release == "buster" or release == "bullseye":
            sources.write("deb %s/ %s main non-free contrib\n" % (online_repository, release))
            sources.write("deb-src %s/ %s main non-free contrib\n" % (online_repository, release))
            sources.write("\n")
            sources.write("## Major bug fix updates produced after the final release of the\n")
            sources.write("## distribution.\n")
            sources.write("deb %s/ %s-updates main non-free contrib\n" % (online_repository, release))
            sources.write("deb-src %s/ %s-updates main non-free contrib\n" % (online_repository, release))
            sources.write("\n")
            if "buster" in release:
                sources.write("deb http://security.debian.org/ %s/updates main non-free contrib\n" % (release))
                sources.write("deb-src http://security.debian.org/ %s/updates main non-free contrib\n" % (release))
            else:
                sources.write("deb http://security.debian.org/debian-security %s-security main contrib non-free\n" % (release))
                sources.write("deb-src http://security.debian.org/debian-security %s-security main contrib non-free\n" % (release))
        else:
            sources.write("deb %s/ %s main non-free non-free-firmware  contrib\n" % (online_repository, release))
            sources.write("deb-src %s/ %s main non-free non-free-firmware contrib\n" % (online_repository, release))
            sources.write("\n")
            sources.write("## Major bug fix updates produced after the final release of the\n")
            sources.write("## distribution.\n")
            sources.write("deb %s/ %s-updates main non-free non-free-firmware contrib\n" % (online_repository, release))
            sources.write("deb-src %s/ %s-updates main non-free non-free-firmware contrib\n" % (online_repository, release))
            sources.write("\n")
            sources.write("deb http://security.debian.org/debian-security %s-security main contrib non-free non-free-firmware \n" % (release))
            sources.write("deb-src http://security.debian.org/debian-security %s-security main contrib non-free non-free-firmware \n" % (release))

if proxy:
    logger.notice(u"Updating apt.conf")
    with open( os.path.join(target, 'etc', 'apt', 'apt.conf'), 'w' ) as apt:
        apt.write('Acquire::http::Proxy "%s";\n' % (proxy))

    logger.notice(u"updating environment variables")
    with open( os.path.join(target, 'etc', 'profile.d', 'proxy.sh'), 'w' ) as proxysh:
        proxysh.write('export http_proxy=%s/\n' % (proxy))
        proxysh.write('export https_proxy=%s/\n' % (proxy))
        proxysh.write('export no_proxy=%s/\n' % (no_proxy))


logger.notice(u"We are on a predictable network interface names bootimage")
for device, lease in leasesFiles.items():
    networkdevice = device

logger.notice(u"Creating /etc/netplan/01-netcfg.yaml ....")
os.makedirs(os.path.join(target, 'etc', 'netplan'), exist_ok=True)
with open( os.path.join(target, 'etc', 'netplan', '01-netcfg.yaml'), 'w' ) as netcfg:
    netcfg.write("network:\n")
    netcfg.write("  version: 2\n")
    netcfg.write("  renderer: networkd\n")
    netcfg.write("  ethernets:\n")
    if addAllNetworkDevices:
        for device in list(filter(None, execute("ls /sys/class/net | grep '^e'"))):
            netcfg.write("    %s:\n" % device)
            netcfg.write("      dhcp4: true\n")
            netcfg.write("      dhcp6: true\n")
            if netplan_use_mac_identifier:
                netcfg.write("      dhcp-identifier: mac\n")
    else:
        netcfg.write("    %s:\n" % usedNetworkDevice.get('device'))
        netcfg.write("      dhcp4: true\n")
        netcfg.write("      dhcp6: true\n")
        if netplan_use_mac_identifier:
            netcfg.write("      dhcp-identifier: mac\n")

logger.notice(u"Creating /etc/hostname....")
with open( os.path.join(target, 'etc', 'hostname'), 'w' ) as etchostname:
    etchostname.write("%s\n" % hostname)

if encrypt_lvm:
    logger.notice(u"Creating /etc/crypttab....")
    with open( os.path.join(target, 'etc', 'crypttab'), 'w' ) as crypttab:
        crypttab.write('lukslvm UUID=%s none luks\n' % lukslvmPartitionUUID)

if inUefi:
    logger.notice(u"Cleanup uefi boot entries ....")
    deleteAllEntriesFromEFIBootmgr('Ubuntu')
    #deleteAllEntriesFromEFIBootmgr('Windows Boot Manager')
    deleteAllEntriesFromEFIBootmgr('opsitempwinpe')
    execute('/bin/efibootmgr -v')

logger.notice(u"Mounting .......")
execute("mount -o bind /proc %s/proc" % target)
execute("mount -o bind /dev %s/dev" % target)
execute("mount -o bind /sys %s/sys" % target)
execute("mount -o bind /run %s/run" % target)
if inUefi:
    execute("mount --bind /sys/firmware/efi/efivars %s/sys/firmware/efi/efivars" % target)
os.putenv("LD_PRELOAD", "")

cmds = []
cmds.append("export DEBIAN_PRIORITY=critical")
cmds.append("export DEBIAN_FRONTEND=noninteractive")
cmds.append("apt-get update")
cmds.append("locale-gen")
cmds.append("apt --yes --fix-broken install")
cmds.append("apt-get --yes --force-yes install console-common console-setup console-data grub-pc locales keyboard-configuration wget")
if release == "bookworm":
    cmds.append("apt-get --yes --force-yes install gdisk efibootmgr python3-software-properties")
else:
    cmds.append("apt-get --yes --force-yes install gdisk efibootmgr python-software-properties")
cmds.append("apt-get --yes --force-yes install aptitude tasksel x11-xkb-utils debconf-utils netplan.io")
if "ubuntu" in productId:
    cmds.append("apt-get --yes --force-yes install linux-generic linux-firmware")
else:
    cmds.append("apt-get --yes --force-yes install linux-image-amd64 firmware-linux")
if use_lvm:
    cmds.append("apt-get --yes --force-yes install lvm2")
    cmds.append("echo dm-mod >> /etc/modules")
    cmds.append("echo dm-snapshot >> /etc/modules")
    cmds.append("echo dm-mirror >> /etc/modules")
    cmds.append("echo dm-mod >> /etc/initramfs-tools/modules")
    cmds.append("echo dm-snapshot >> /etc/initramfs-tools/modules")
    cmds.append("echo dm-mirror >> /etc/initramfs-tools/modules")
if encrypt_lvm:
    cmds.append("apt-get --yes --force-yes install cryptsetup cryptsetup-initramfs")
    cmds.append("echo dm-crypt >> /etc/modules ")
    cmds.append("echo ohci_pci >> /etc/initramfs-tools/modules")
    cmds.append("echo aes >> /etc/initramfs-tools/modules")
    cmds.append("echo aes_i586 >> /etc/initramfs-tools/modules")
    cmds.append("echo aes_x86_64 >> /etc/initramfs-tools/modules")
    cmds.append("echo aes_generic >> /etc/initramfs-tools/modules")
    cmds.append("echo dm-crypt >> /etc/initramfs-tools/modules")
    cmds.append("echo sha256 >> /etc/initramfs-tools/modules")
    cmds.append("echo sha256_generic >> /etc/initramfs-tools/modules")
    cmds.append("echo lrw >> /etc/initramfs-tools/modules")
    cmds.append("echo xts >> /etc/initramfs-tools/modules")
    cmds.append("echo crypto_blkcipher >> /etc/initramfs-tools/modules")
    cmds.append("echo gf128mul >> /etc/initramfs-tools/modules")

cmds.append("update-initramfs -k all -u")
if usedNetworkDevice.get('device').find('eth') != -1:
    netifnames = '0'
else:
    netifnames = '1'
if release == "focal" or release == "jammy" or release == "noble":
    cmds.append("sed -i s/splash/nosplash/g /etc/default/grub")
if inUefi:
    #http://wiki.ubuntuusers.de/GRUB_2/Reparatur
    cmds.append("mkdir -p /boot/efi")
    cmds.append("mount %s /boot/efi" % getPartitionDevicePath(disk.device, 1))
    cmds.append("mount")
    cmds.append("apt-get --yes --force-yes install grub-efi")
    cmds.append("apt-get --yes --force-yes install shim-signed")
    if encrypt_lvm:
        cmds.append('sed -i "/GRUB_CMDLINE_LINUX_DEFAULT/d" /etc/default/grub')
        cmds.append('echo \'GRUB_CMDLINE_LINUX_DEFAULT="noplymouth kopt=root=%s "\' >> /etc/default/grub' % (systemPartitionDevicePath))
    cmds.append("update-grub")
    cmds.append("grub-install %s" % (disk.device))
    cmds.append("grub-install --recheck %s" % (disk.device))
    cmds.append("umount %s" % getPartitionDevicePath(disk.device, 1))
    cmds.append("mount")
else:
    cmds.append("apt-get --yes --force-yes install grub-pc")
    cmds.append("dpkg-reconfigure grub-pc")
    if encrypt_lvm:
        cmds.append('sed -i "/GRUB_CMDLINE_LINUX_DEFAULT/d" /etc/default/grub')
        cmds.append('echo \'GRUB_CMDLINE_LINUX_DEFAULT="kopt=root=%s"\' >> /etc/default/grub' % (systemPartitionDevicePath))
    cmds.append("update-grub")
    cmds.append("grub-install --recheck %s" % (disk.device))
cmds.append("if [ -f /etc/localtime.bak ] ; then rm /etc/localtime.bak ; fi")
cmds.append("mv /etc/localtime /etc/localtime.bak")
cmds.append("ln -s /usr/share/zoneinfo/%s /etc/localtime" % productProperties.get('timezone', 'Europe/Berlin'))
cmds.append("KEYMAP=`find /usr/share/keymaps/i386/ -name %s.kmap.gz`"  % productProperties.get('console_keymap', 'de-latin1-nodeadkeys'))
cmds.append("echo Will install keymap file $KEYMAP")
cmds.append("install-keymap $KEYMAP")
cmds.append("cp $KEYMAP /etc/console-setup/cached.kmap.gz")
#cmds.append("cp $KEYMAP /etc/console/boottime.kmap.gz")
cmds.append("cp $KEYMAP /etc/console-setup/")
cmds.append("loadkeys %s" % productProperties.get('console_keymap', 'de-latin1-nodeadkeys'))
cmds.append("setupcon")
cmds.append("chmod ugu+rwx /tmp ; ls -ld /tmp")
cmds.append("update-initramfs -k all -u")
cmds.append("echo root:%s | chpasswd" % productProperties.get('root_password', 'linux123'))
cmds.append("useradd  --groups sudo -m -s /bin/bash %s" % productProperties.get('user_name', 'user'))
cmds.append("echo %s:%s | chpasswd" % (productProperties.get('user_name', 'user'), productProperties.get('user_password', 'linux123')))
cmds.append("apt-get --yes --force-yes install openssh-server openssh-client")
cmds.append("systemctl enable ssh")
cmds.append("netplan apply")

if isOpsi43:
    show_message(u"Base configuration / bootloader /console...", 60)
scriptMessageSubject.setMessage("Base configuration / bootloader /console...")
logger.notice(u"Base configuration / bootloader /console...")
doChrooted(target, cmds, proxy, no_proxy)

additional_grub_parameter = ""
if productProperties.get('additional_grub_parameter'):
    additional_grub_parameter = productProperties.get('additional_grub_parameter') 

newLines = []
with open('%s/etc/default/grub' % target, 'r+') as grubDefault:
    for line in grubDefault:
        if line.strip().startswith('GRUB_CMDLINE_LINUX_DEFAULT'):
            newLines.append('%s net.ifnames=%s %s"\n' % (line.rstrip()[:-1], netifnames, additional_grub_parameter))
            continue
        newLines.append(line)

with open('%s/etc/default/grub' % target, 'w') as grubDefault:
    grubDefault.writelines(newLines)

cmds = []
cmds.append("update-grub")
doChrooted(target, cmds, proxy, no_proxy)

scriptMessageSubject.setMessage("Base configuration / localization / Generating locales ... ")
logger.notice(u"Base configuration / localization / Generating locales ...")
language = productProperties.get('language','de')
if language == 'de':
    locale="de_DE.UTF-8"
    keymap="de-latin1-nodeadkeys"
if language == 'en':
    locale="en_US.UTF-8"
    keymap="us"
if language == 'fr':
    locale="fr_FR.UTF-8"
    keymap="fr"
if language == 'dk':
    locale="da_DK.UTF-8"
    keymap="dk"
if language == 'it':
    locale="it_IT.UTF-8"
    keymap="it"

cmds = []
# Generate and set locales
# This will lead to generating all locales:
#cmds.append("cat /usr/share/i18n/SUPPORTED > /etc/locale.gen")
cmds.append('cat /usr/share/i18n/SUPPORTED |grep "en_US.UTF-8\|%s" > /etc/locale.gen' % locale)
cmds.append('echo "locales locales/default_environment_locale      select  %s" | debconf-set-selections' % locale)
cmds.append("debconf-show locales")
cmds.append("dpkg-reconfigure --priority=critical locales")
cmds.append('cat /etc/environment ')
cmds.append("update-locale LANG=%s" % locale)
cmds.append('cat /etc/default/locale ')
cmds.append('echo "LANG=%s"        >  /etc/environment' % locale)
cmds.append('echo "LANGUAGE=%s"    >> /etc/environment' % language)
cmds.append('echo "LC_ALL=%s"    >> /etc/environment' % locale)
cmds.append('echo "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games:/usr/local/games" >> /etc/environment')
cmds.append('cat /etc/environment ')
if "ubuntu" in productId:
    cmds.append('echo "LANG=%s"        >  /etc/default/locale' % locale)
else:
    cmds.append("update-locale LANG=%s" % locale)
cmds.append('cat /etc/default/locale ')
cmds.append("export LANGUAGE=%s"  % locale)
cmds.append("export LANG=%s"  % locale)
cmds.append("export LC_ALL=%s"  % locale)
cmds.append("install-keymap %s" % keymap)
# set keyboard type and layout
cmds.append("export DEBIAN_FRONTEND=noninteractive")
cmds.append('echo "keyboard-configuration  keyboard-configuration/xkb-keymap       select  %s" | debconf-set-selections' % language)
cmds.append('echo "keyboard-configuration  keyboard-configuration/layoutcode       string  %s" | debconf-set-selections' % keymap)
cmds.append("debconf-show keyboard-configuration")
cmds.append("dpkg-reconfigure --priority=critical keyboard-configuration")
cmds.append('echo "console-common  console-data/keymap/full        select  %s" | debconf-set-selections' % keymap)
cmds.append('echo "console-data    console-data/keymap/full        select  %s" | debconf-set-selections' % keymap)
cmds.append("debconf-show console-data")
cmds.append("dpkg-reconfigure --priority=critical console-data")
cmds.append("if [ -f /etc/default/keyboard ]  ; then cp /etc/default/keyboard /etc/default/keyboard.org ; fi")
if "ubuntu" in productId:
    # install language packs
    cmds.append("apt-get --yes --force-yes install language-pack-%s" % language)
    cmds.append("apt-get --yes --force-yes install language-pack-%s-base" % language)
doChrooted( target, cmds, proxy, no_proxy)

cmds = []

try:
    tf = TextFile("%s/etc/default/keyboard" % target)
    lines = tf.readlines()
    if lines:
        for i in range(len(lines)):
            lines[i] = lines[i].strip()
            if lines[i].startswith("XKBLAYOUT="):
                lines[i] = 'XKBLAYOUT="%s"' % language
                break
        tf.open(mode="wb")
        tf.writelines(lines)
        tf.close()
except Exception as e:
    logger.error(e)

if productProperties.get('additional_packages'):
    cmds.append("apt-get --yes --force-yes install %s" % productProperties.get('additional_packages'))

scriptMessageSubject.setMessage("Additional Packages...")
logger.notice(u"Additional Packages...")
doChrooted( target, cmds, proxy, no_proxy)


if install_opsi_client_agent:
    if isOpsi43:
        show_message(u"Installing opsi-linux-client-agent", 70)
    cmds = []
    cmds.append("export DEBIAN_FRONTEND=noninteractive")
    cmds.append("apt-get --yes --force-yes install lsb-release")
    cmds.append("apt-get --yes --force-yes install netcat-openbsd")
    cmds.append("apt-get --yes --force-yes install smbfs")
    cmds.append("apt-get --yes --force-yes install cifs-utils")
    cmds.append("apt-get --yes --force-yes install xterm")
    cmds.append("apt-get --yes --force-yes install python librsync1")
    cmds.append("apt-get --yes --force-yes install dnsutils lsof")
    cmds.append("apt-get --yes --force-yes install pkg-config openssl")
    cmds.append("apt-get --yes --force-yes install wget openssh-server")
    cmds.append("apt-get --yes --force-yes install vim librsync-dev")
    cmds.append("systemctl enable ssh")
    scriptMessageSubject.setMessage("Installing opsi-linux-client dependencies...")
    logger.notice(u"Installing opsi-linux-client dependencies...")
    doChrooted(target, cmds, proxy, no_proxy)
    execute('/bin/sleep 2')

    depotAddress = depotUrl.split('/')[2].replace(':4447','')
    client_id = fqdn
    client_key = pckey
    logger.notice(u"Copy opsi-client-agent....")
    cmds = []
    cmds.append("mkdir -p /var/tmp/opsi-linux-client-agent")
    cmds.append("wget -r -nH --no-check-certificate --cut-dirs=2 --no-parent --http-user {0} --http-passwd {1} --reject=\"index.html*\" https://{2}:4447/depot/opsi-linux-client-agent/ -P /var/tmp/opsi-linux-client-agent".format(client_id, client_key, depotAddress))
    cmds.append("chmod -R 777 /var/tmp/opsi-linux-client-agent")
    if project_quota:
        cmds.append("wget -r -nH --no-check-certificate --cut-dirs=3 --no-parent --http-user {0} --http-passwd {1} --reject=\"index.html*\" https://{2}:4447/depot/{3}/opsi/postinst.d/ -P /var/tmp/opsi-linux-client-agent/files".format(client_id, client_key, depotAddress, productId))
        cmds.append("find /var/tmp/opsi-linux-client-agent/files/postinst.d -name \"*.sh\" -exec chmod -x {} +")
    cmds.append("echo 'CLIENT_ID={0}' >> /var/tmp/opsi-linux-client-agent/files/postinst.d/postinst.env".format(client_id))
    cmds.append("echo 'CLIENT_KEY={0}' >> /var/tmp/opsi-linux-client-agent/files/postinst.d/postinst.env".format(client_key))
    cmds.append("echo 'SERVICE_ADDRESS=https://{0}:4447' >> /var/tmp/opsi-linux-client-agent/files/postinst.d/postinst.env".format(depotAddress))
    cmds.append("wget -r -nH --no-check-certificate --cut-dirs=3 --no-parent --http-user {0} --http-passwd {1} --reject=\"index.html*\" https://{2}:4447/depot/{3}/opsi/postinst.sh -P /var/tmp/opsi-linux-client-agent/files/".format(client_id, client_key, depotAddress, productId))
    cmds.append("chmod +x /var/tmp/opsi-linux-client-agent/files/postinst.sh")
    cmds.append("wget -r -nH --no-check-certificate --cut-dirs=2 --no-parent --http-user {0} --http-passwd {1} --reject=\"index.html*\" https://{2}:4447/depot/{3}/olca-install.service -P /etc/systemd/system/".format(client_id, client_key, depotAddress, productId))
    cmds.append("sed -i s/service_ip_address/{0}/g /etc/systemd/system/olca-install.service".format(depotAddress))
    cmds.append("systemctl enable olca-install.service")

    scriptMessageSubject.setMessage("preparing opsi linux client agent...")
    logger.notice(u"preparing opsi linux client agent....")
    doChrooted(target, cmds, proxy, no_proxy)
    execute('/bin/sleep 5')

if inUefi:
    execute('/bin/efibootmgr -v')

handle_wget_and_execute()
unmountDisk()
execute('umount -l %s' % target)
if project_quota:
    logger.notice("Quota specific commands")
    execute("tune2fs -Q prjquota %s && tune2fs -E mount_opts=prjquota %s" % (systemPartitionDevicePath,systemPartitionDevicePath))
handle_setupAfterInstall()

if olimode:
    if install_opsi_client_agent:
        productsToDelete = backend.productOnClient_getObjects(clientId=clientId)
        if productsToDelete:
            backend.productOnClient_deleteObjects(productsToDelete)

    # write log to data partition
    imageDir = '/mntimg'
    filesystem = 'ext4'
    product = 'opsi-local-image-ubuntu'

    # mount the image (backup) file system
    # check the mount point
    checkCreateDir(imageDir)
    # check the backup partition
    disk = disks[backupdiskindex]
    logger.notice(u"Using disk device %s" % (disk.device))

    try:
        testPartitionEx(disk.device, backupPartitionNumber, useGpt)
    except:
        raise Exception(u"Could not find partition %d. - Please restart this product (opsi-local-image-prepare)" % backupPartitionNumber)

    partitionLabel = partitionHasLabel(disk.device, backupPartitionNumber)
    if (partitionLabel == backupPartitionLabel):
        logger.notice(u"%s exists as expected - fine" % (getPartitionDevicePath(disk.device, backupPartitionNumber)))
        logger.notice(u"%s created and formated" % (getPartitionDevicePath(disk.device, backupPartitionNumber)))
    else:
        logger.error(u"%s exists not as expected with label %s - not ok - giving up" % (getPartitionDevicePath(disk.device, backupPartitionNumber), backupPartitionLabel))
        scriptMessageSubject.setMessage(u"%s exists not as expected with label %s - not ok - giving up" % (getPartitionDevicePath(disk.device, backupPartitionNumber), backupPartitionLabel))
        raise Exception(u"Wrong partition table - Please run the opsi product 'opsi-local-image-prepare' to prepare this disk ! ")

    # check if mount point is in use
    checkClearMountpoint(imageDir, disk.device, backupPartitionNumber)
    # check if backup partition is in use
    checkClearPartition(imageDir, disk.device, backupPartitionNumber)
    # mount backup partition to mount point
    checkMount(imageDir, disk.device, backupPartitionNumber)


    if goOn:
        if backupAfterInstall:
            logger.notice(u"Setting opsi-local-image-backup-starter to once")
            # set opsi-local-image-backup-starter to once
            backend.setProductActionRequest(productId='opsi-local-image-backup-starter', clientId=clientId, actionRequest='once')
            # Setting defaults for opsi-local-image-restore
            logger.notice(u"Setting defaults for opsi-local-image-restore to empty")
            imageFilesList = []
            backend.productPropertyState_create(productId =  'opsi-local-image-backup',propertyId = 'imagefile',objectId = clientId,values = imageFilesList)

        if setupAfterInstall:
            for setupProduct in setupAfterInstall:
                setupProduct = setupProduct.strip()
                if setupProduct != '':
                    logger.notice(u"Set to setup %s" % (setupProduct) )
                    backend.setProductActionRequestWithDependencies(productId=setupProduct, clientId=clientId, actionRequest='setup')


        # write meta info to masterlog
        filesystem = 'ext4'
        logstr = time.asctime( time.localtime(time.time()) )
        logstr = logstr + ', product=%s, filesystem=%s' % (productId, filesystem)
        logFile = imageDir+'/master.log'
        if not os.path.exists(logFile):
            f = open(logFile, 'w' )
        else:
            f = open(logFile, 'a' )
        f.write(logstr)
        f.close()
        # show
        logger.notice(u"Here comes the master log:")
        execute(u'cat %s' % (logFile) )
# Set product installation status
productOnClient.setInstallationStatus('installed')
productOnClient.setActionProgress('')
productOnClient.setActionResult('successful')
productOnClient.setLastAction('setup')

# Reboot
reboot()
